//
//  MirroredWindow.m
//  WindowCaptureDemo
//
//  Created by David Gelhar on 11/2/12.
//  Copyright (c) 2012 __MyCompanyName__. All rights reserved.
//

#import "MirroredWindow.h"
#import "NSImage+ImageGrabber.h"
#import "CGSPrivate.h"

@implementation MirroredWindow

@synthesize sourceWindowID,sourceWindowDict;

// Create a mirrored window from the top window on the screen
+(MirroredWindow *) windowFromWindowDict:(NSDictionary *)windowDict {
    
    // match frame of new window to bounds of source
    MirroredWindow *window = [[self alloc] 
                           initWithContentRect:[MirroredWindow flippedWindowBounds:windowDict]
                           styleMask:0 //NSBorderlessWindowMask 
                           backing:NSBackingStoreBuffered 
                           defer:NO];

    if (window != nil) {
 
        // Configure window.
	[window setReleasedWhenClosed:YES];
	[window setHidesOnDeactivate:NO];
	[window setCanHide:NO];
	[window setCollectionBehavior:NSWindowCollectionBehaviorCanJoinAllSpaces];
	[window setIgnoresMouseEvents:YES];
	[window setLevel:NSScreenSaverWindowLevel];
        
        window.sourceWindowDict = windowDict;// Hang on to the full dictionary of info from CGWindowList
        // extract windownumber for convenience
        window.sourceWindowID = [[windowDict objectForKey:@"kCGWindowNumber"] intValue];
    
        if (! [window refresh]) {
            NSLog(@"Initial refresh for source window %d failed",  window.sourceWindowID);
        }
        
        [window makeKeyAndOrderFront:nil];
        
    }
    
    return window;
    
}

// Pick up latest image
- (BOOL) refresh {
    OSStatus err;
    
    // Verify that source window is still there
    NSDictionary *windowDict = [MirroredWindow findWindow:sourceWindowID];
    if (windowDict == nil) {
        return NO;
    }
    NSRect  srcBounds = [MirroredWindow flippedWindowBounds:windowDict];
    
    if (!NSEqualRects([self frame], srcBounds)) {   // source window moved?
        [self setFrame:srcBounds display:YES];  // track source window
    }
    
    // Grab image from source window
    NSImage *newImage = [NSImage imageWithWindow:sourceWindowID];
    NSImageView *imageView = [[NSImageView alloc] init];
    
    [imageView setImage:newImage];
    
    // mirror new image horizontally
    CGAffineTransform transform;
    // start with source window transform matrix 
    err = CGSGetWindowTransform(_CGSDefaultConnection(), (CGSWindow)sourceWindowID, &transform);
    if (err != noErr) {         // will fail if source window goes away
        return NO;
    }
    
    // step 1 - slide left so right edge of image is on x=0
    transform = CGAffineTransformTranslate(transform, self.frame.origin.x+self.frame.size.width, 0.0);
    
    // horizontal flip; now x = [0..frame.size.width
    transform = CGAffineTransformScale(transform, -1.0, 1.0);  
    
    // slide flipped image back into place
    transform = CGAffineTransformTranslate(transform, -self.frame.origin.x, 0); 
    // set copied image as content of new window
    [self setContentView:imageView];
    
    CGSSetWindowTransform(_CGSDefaultConnection(), (CGSWindow)[self windowNumber], transform);    
    
    return YES;
}

// Utility - find window information for a specific CGWindowID
+ (NSDictionary *)findWindow:(CGWindowID) windowID {
    
    // Get list of windows (from all apps, topmost first)
    NSMutableArray *windows = (__bridge NSMutableArray *)CGWindowListCopyWindowInfo(kCGWindowListOptionOnScreenAboveWindow | kCGWindowListOptionIncludingWindow, windowID);
    
    // find first window in layer 0
    for (NSDictionary *window in windows) {
        NSNumber *windowNumber = [window objectForKey:@"kCGWindowNumber"];
        if ([windowNumber intValue] == windowID) { 
            return window;           
        }
    }
    return nil;
}

// Utility - get window bounds from dictionary, flip coordinates
+ (NSRect) flippedWindowBounds:(NSDictionary *)windowDict {
    // determine bounds of source window
    CGRect srcbounds;
    CGRectMakeWithDictionaryRepresentation((__bridge CFDictionaryRef)[windowDict objectForKey:@"kCGWindowBounds"], &srcbounds);
    NSRect bounds = NSRectFromCGRect(srcbounds); // convert CGRect to NSRect

    return [MirroredWindow flipVerticalCoordinates:bounds]; // and flip the coordinate system

}

// Utility - flip coordinate system of an NSRect to have Y=0 at lower left (NS* operations)
+ (NSRect) flipVerticalCoordinates:(NSRect) frame {
    // Compute height of main screen for coordinate flip
    CGFloat screenHeight = [[[NSScreen screens] objectAtIndex:0] frame].size.height;

    // flip coordinates to have Y=0 at lower left (NS* operations)
    frame.origin.y = screenHeight - frame.size.height - frame.origin.y;

    return frame;
}

@end
